/*
 * Copyright (C) 2014-2019 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __FW_DRV_H__
#define __FW_DRV_H__

#include <linux/cdev.h>
#include <linux/semaphore.h>
#include <linux/ktime.h>

#include "../include/fw-board.h"

#define PCI_MAJOR   0   /* dynamic major by default */
#define PCI_DEVS    8
#define IRQ_TIMEOUT_MS 5000 /* 5s */
#define IRQ_VECS_MAX 4

// PLX UserIO, Serial EEPROM, and
extern int pci_major;
extern int pci_devs;

typedef enum {
    MY_BOARD = 0,
} board_t;

typedef struct _INFO_IO_SPACE
{
    bool      bSpaceMapped;
    uintptr_t nSpaceRange;
    uintptr_t physicalAddress;
    void*     pSpaceBase;
} _INFO_IO_SPACE, *_PINFO_IO_SPACE;

struct pci_priv_device {
    _INFO_IO_SPACE      IoSpacePciPlx;
    _INFO_IO_SPACE      IoSpacePciFipWatcher;
    uintptr_t           IoPortPciPlxAddress;
    uintptr_t           IoPortPciPlxSize;

    int                 is_open;    /* 0: free - 1: busy */
    struct list_head    link;       /* Double linked list */
    struct pci_dev      *pci_dev;   /* PCI device */
    struct device       *dev;
    board_t             type;       /* Board type */
    u32                 minor;      /* Minor number */
    struct semaphore    sem;        /* Mutual exclusion */
    struct cdev         cdev;
    int                 int_flag;
    wait_queue_head_t   int_mutex;
    spinlock_t          int_lock;
    ktime_t             prectime;

    str_exo_fwdev       infos;
    str_irq             interrupts;
    bool                has_msi_cap;
    int                 nb_irq_vecs;
    u32                 irq_handled_counter;
    u32                 irq_none_handled_counter;
};

extern struct file_operations pcifw_fops;

#endif
